﻿namespace SampleSmsChannel.Plugins
{
    using Microsoft.Xrm.Sdk;
    using Microsoft.Xrm.Sdk.Extensions;
    using SampleSmsChannel.Plugins;
    using System;
    using System.Collections.Generic;

    // This is a plugin for sending delivery report to Channel Definition Delivery report API
    // which should be called by a proxy service that processes incoming notifications from serice provider
    public class DeliveryReportPlugin : IPlugin
    {
        public void Execute(IServiceProvider serviceProvider)
        {
            var tracingService = serviceProvider.Get<ITracingService>();
            tracingService.Trace("Executing delivery report plugin");
            var pluginExecutionContext = serviceProvider.Get<IPluginExecutionContext>();
            var payload = pluginExecutionContext.InputParameters["payload"] as string;
            tracingService.Trace(payload);
            var organizationService = serviceProvider.Get<IOrganizationServiceFactory>().CreateOrganizationService(null);

            // The payload sent to this plugin can be in any format defined by the implementation. This is a sample payload contract
            var providerDeliveryReport = JsonUtils.Deserialize<ProviderDeliveryReport>(payload);

            // This is the contract accepted by Channel Definition Delivery report API
            var deliveryReport = new DeliveryReport()
            {
                ChannelDefinitionId = Guid.Parse("d856474c-904b-4b72-841e-b3d8a14a87c4"),
                From = providerDeliveryReport.From,
                MessageId = providerDeliveryReport.MessageSid,

                // Request Id needs to be correlated by the custom channel implementation if not provided by the sending provider
                RequestId = providerDeliveryReport.RequestId,
                Status = "Delivered",
                OrganizationId = pluginExecutionContext.OrganizationId.ToString(),
                StatusDetails = new Dictionary<string, object>()
            };

            var notificatonPayload = JsonUtils.Serialize(deliveryReport);
            tracingService.Trace("Notification payload: {0}", notificatonPayload);

            // Channel Definition Delivery report API execution
            var response = organizationService.Execute(new OrganizationRequest("msdyn_D365ChannelsNotification")
            {
                Parameters = 
                {
                    { "notificationPayLoad", notificatonPayload }
                }
            });

            // Using it for debugging purpose
            pluginExecutionContext.OutputParameters["response"] = response.Results["responseMessage"];
        }
    }
}
