﻿namespace SampleSmsChannel.Plugins
{
    using Microsoft.Xrm.Sdk;
    using Microsoft.Xrm.Sdk.Extensions;
    using SampleSmsChannel.Plugins.ChannelContracts;
    using System;

    // Sample Plugin for outbound calls. It doesn't send any real messages, just logs the payload
    public class OutboundPlugin : IPlugin
    {
        public void Execute(IServiceProvider serviceProvider)
        {
            var tracingService = serviceProvider.Get<ITracingService>();
            tracingService.Trace("Executing outbound sample channel plugin");
            var pluginExecutionContext = serviceProvider.Get<IPluginExecutionContext>();

            // "payload" attribute is required by the contract
            var payload = pluginExecutionContext.InputParameters["payload"] as string;
            tracingService.Trace(payload);

            // This is the contract for outbound payload
            var payloadObject = JsonUtils.Deserialize<Payload>(payload);

            var responseObject = new Response()
            {
                ChannelDefinitionId = payloadObject.ChannelDefinitionId,
                // Set Message id if provided by the sending provider
                MessageId = Guid.NewGuid().ToString(),
                RequestId = payloadObject.RequestId,
                // Refer to status list from the documentation
                Status = "Sent",
                StatusDetails = null
            };

            // "response" attribute is required by the contract
            pluginExecutionContext.OutputParameters["response"] = JsonUtils.Serialize(responseObject);
        }
    }
}
